@@ -40,6 +40,7 @@ except ImportError:
     syslog = None
 
 from oslo_config import cfg
+from oslo_utils import eventletutils
 from oslo_utils import importutils
 from oslo_utils import units
 
@@ -231,7 +232,7 @@ def _load_log_config(log_config_append):
             logging.config.fileConfig(log_config_append,
                                       disable_existing_loggers=False)
             _load_log_config.old_time = new_time
-    except (configparser.Error, KeyError, os.error) as exc:
+    except (configparser.Error, KeyError, os.error, RuntimeError) as exc:
         raise LogConfigError(log_config_append, str(exc))
 
 
@@ -265,8 +266,25 @@ def register_options(conf):
     conf.register_mutate_hook(_mutate_hook)
 
 
-def setup(conf, product_name, version='unknown'):
+def _fix_eventlet_logging():
+    """Properly setup logging with eventlet on native threads.
+
+    Workaround for: https://github.com/eventlet/eventlet/issues/432
+    """
+
+    # If eventlet was not loaded before call to setup assume it's not used.
+    if eventletutils.is_monkey_patched('thread'):
+        import eventlet.green.threading
+        from oslo_log import pipe_mutex
+        logging.threading = eventlet.green.threading
+        logging._lock = logging.threading.RLock()
+        logging.Handler.createLock = pipe_mutex.pipe_createLock
+
+
+def setup(conf, product_name, version='unknown', *, fix_eventlet=True):
     """Setup logging for the current application."""
+    if fix_eventlet:
+        _fix_eventlet_logging()
     if conf.log_config_append:
         _load_log_config(conf.log_config_append)
     else:
@@ -387,7 +405,10 @@ def _setup_logging_from_conf(conf, project, version):
         log_root.addHandler(streamlog)
 
     if conf.use_journal:
-        journal = handlers.OSJournalHandler()
+        if syslog is None:
+            raise RuntimeError("syslog is not available on this platform")
+        facility = _find_facility(conf.syslog_log_facility)
+        journal = handlers.OSJournalHandler(facility=facility)
         log_root.addHandler(journal)
 
     if conf.use_eventlog:
@@ -412,7 +433,6 @@ def _setup_logging_from_conf(conf, project, version):
         log_root.addHandler(handler)
 
     if conf.use_syslog:
-        global syslog
         if syslog is None:
             raise RuntimeError("syslog is not available on this platform")
         facility = _find_facility(conf.syslog_log_facility)
